--
-- ThreatBar.lua
-- Copyright 2008-2021 Johannes Rydh
--
-- Permission is hereby granted, free of charge, to any person obtaining
-- a copy of this software and associated documentation files (the
-- "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish,
-- distribute, sublicense, and/or sell copies of the Software, and to
-- permit persons to whom the Software is furnished to do so, subject to
-- the following conditions:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
-- LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-- OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-- WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--

local max = _G.max;
local UnitExists = _G.UnitExists;
local UnitIsUnit = _G.UnitIsUnit;
local UnitDetailedThreatSituation = _G.UnitDetailedThreatSituation;
local GetNumGroupMembers = _G.GetNumGroupMembers;
local GetNumSubGroupMembers = _G.GetNumSubGroupMembers;

ThreatBar = LibStub("AceAddon-3.0"):NewAddon( "ThreatBar", "AceEvent-3.0" );

function ThreatBar:OnInitialize()
   self.frame = self:CreateFrame();
end

function ThreatBar:CreateFrame()
   local f = CreateFrame( "Frame", "ThreatBarFrame", UIParent,
                          "SecureFrameTemplate,BackdropTemplate" );
   f:SetAttribute( "unit", "target" );
   RegisterStateDriver( f, "visibility",
                        "[combat,harm,exists,nodead] show; hide" );

   f:SetBackdrop( { bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
                    tile = true, tileSize = 16, nil,
                    insets = { left = 4, right = 4, top = 4, bottom = 4 } } );
   f:SetBackdropColor( 0, 0, 0, 0.5 );

   f:SetWidth( 61 );
   f:SetHeight( 320 );
   f:SetMinResize( 41, 120 );
   f:SetMaxResize( 61, 320 );
   f:EnableMouse( true );
   f:SetMovable( true );
   f:SetResizable( true );
   f:SetScript( "OnMouseDown", function()
       if IsAltKeyDown() then
          f:StartMoving();
       elseif IsShiftKeyDown() then
          f:StartSizing("BOTTOMRIGHT");
       end
   end );
   f:SetScript( "OnMouseUp", function() f:StopMovingOrSizing() end );
   f:SetScript( "OnSizeChanged", function() self:Update() end );
   if not f:GetLeft() then f:SetPoint( "CENTER" ); end

   f.otherbar = f:CreateTexture( nil, "OVERLAY" );
   f.otherbar:SetTexture( "Interface\\AddOns\\ThreatBar\\charcoal" );
   f.otherbar:SetTexCoord( 0, 0, 1, 0, 0, 1, 1, 1 );
   f.otherbar:SetWidth( 10 );
   f.otherbar:SetHeight( 300 );
   f.otherbar:SetPoint( "BOTTOMLEFT", f, "BOTTOMLEFT", 10, 10 );

   f.playerbar = f:CreateTexture( nil, "OVERLAY" );
   f.playerbar:SetTexture( "Interface\\AddOns\\ThreatBar\\charcoal" );
   f.playerbar:SetTexCoord( 0, 0, 1, 0, 0, 1, 1, 1 );
   f.playerbar:SetHeight( 300 );
   f.playerbar:SetPoint( "BOTTOMLEFT", f.otherbar, "BOTTOMRIGHT", 1, 0 );
   f.playerbar:SetPoint( "BOTTOMRIGHT", f, "BOTTOMRIGHT", -10, 10 );

   f.pct = f:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" );
   f.pct:SetWidth( 30 );
   f.pct:SetHeight( 12 );
   f.pct:SetPoint( "BOTTOM", f.playerbar, "BOTTOM", 0, 3 );

   f.deficit = f:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" );
   f.deficit:SetWidth( 50 );
   f.deficit:SetHeight( 12 );
   f.deficit:SetPoint( "TOP", f, "TOP", 0, -3 );

   return f;
end

function ThreatBar:OnEnable()
   local function upd( event, unitid )
      if unitid == "target" then self:Update(); end
   end
   self:RegisterEvent( "UNIT_THREAT_LIST_UPDATE", upd );
   self:RegisterEvent( "PLAYER_TARGET_CHANGED", "Update" );
end

function ThreatBar:OnDisable()
   self:UnregisterEvent( "UNIT_THREAT_LIST_UPDATE" );
   self:UnregisterEvent( "PLAYER_TARGET_CHANGED" );
end

local function check_threat( unit, m_pct, m_raw, m_threat )
   if UnitExists(unit) and not UnitIsUnit("player", unit) then
      local _, _, pct, raw, threat =
         UnitDetailedThreatSituation( unit, "target" );

      if threat and threat > m_threat then
         return pct, raw, threat;
      end
   end
   return m_pct, m_raw, m_threat;
end

function ThreatBar:Update()
   local isTanking, status, scaledpct, rawpct, threatvalue
      = UnitDetailedThreatSituation( "player", "target" );

   if status == nil or rawpct == 0 then	-- not in the mob's aggro list
      self.frame.otherbar:Hide();
      self.frame.playerbar:Hide();
      self.frame.pct:SetText( "0%" );
      self.frame.deficit:SetText( "n/a" );
      return;
   end

   self.frame.playerbar:Show();

   local maxheight = self.frame:GetHeight() - 20;
   local deficit = 0;

   if status == 0 or status == 1 then
      self.frame.otherbar:Show();
      self.frame.otherbar:SetHeight( floor( maxheight*scaledpct/rawpct ) );
      self.frame.otherbar:SetVertexColor( scaledpct/100, 1-scaledpct/100, 0 );
      self.frame.playerbar:SetHeight( floor( maxheight*scaledpct/100 ) );
      if status == 0 then
         self.frame.playerbar:SetVertexColor( 0.69, 0.69, 0.69 );
      else
         self.frame.playerbar:SetVertexColor( 1, 1, 0.47 );
      end
      self.frame.pct:SetText( ("%d%%"):format( floor(rawpct) ) );

      deficit = floor( threatvalue * ( 1 - 100/rawpct ) / 100 );

   elseif status == 2 or status == 3 then
      -- Find second highest on threat
      local m_pct, m_raw, m_threat = 0, 0, 0;
      if IsInRaid() then
         for i = 1, GetNumGroupMembers() do
            m_pct, m_raw, m_threat =
               check_threat( "raid"..i, m_pct, m_raw, m_threat );
            m_pct, m_raw, m_threat =
               check_threat( "raidpet"..i, m_pct, m_raw, m_threat );
         end
      else
         for i = 1, GetNumSubgroupMembers() do
            m_pct, m_raw, m_threat =
               check_threat( "party"..i, m_pct, m_raw, m_threat);
            m_pct, m_raw, m_threat =
               check_threat( "partypet"..i, m_pct, m_raw, m_threat );
         end
      end
      m_pct, m_raw, m_threat = check_threat( "pet", m_pct, m_raw, m_threat );

      if m_raw > 0 then
         self.frame.otherbar:Show();
         self.frame.otherbar:SetHeight( floor( maxheight*m_pct/100 ) );
         self.frame.otherbar:SetVertexColor( m_pct/100, 1-m_pct/100, 0 );
         self.frame.playerbar:SetHeight( floor( maxheight*m_pct/m_raw ) );
         self.frame.pct:SetText( ("%d%%"):format( floor(m_raw) ) );
      else
         self.frame.otherbar:Hide();
         self.frame.playerbar:SetHeight( maxheight );
         self.frame.pct:SetText( "***" );
      end
      if status == 2 then
         self.frame.playerbar:SetVertexColor( 1, 0.6, 0 );
      else
         self.frame.playerbar:SetVertexColor( 1, 0, 0 );
      end

      deficit = floor( (threatvalue - m_threat)/100 );
   end

   if abs(deficit) >= 1000 then
      self.frame.deficit:SetText( string.format( "%2.1fk", deficit/1000 ) );
   else
      self.frame.deficit:SetText( deficit );
   end
end
